#!/bin/env python3

import sys
import os
import json
import pycurl
import subprocess
from io import BytesIO
from pysqlcipher3 import dbapi2 as sqlite
from datetime import datetime

def runCommand(cmd):
    process = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)

    out = process.stdout.decode('utf-8')
    out = out.replace('\n', '')
    code = process.returncode

    return code, out

def parse_disk(disks, serial=''):
    try:
        for disk in disks:
            children = disk.get('children', None)
            serialTmp = disk.get('serial', None)
            mountpoint = disk.get('mountpoint', '')
            name = disk.get('name', 'no name')
            if serialTmp:
                serial = serialTmp
            serialTmp = ''
            if children:
                serialTmp = parse_disk(children, serial)
            if serialTmp:
                return serialTmp

            if mountpoint == "/mnt/sysroot":
                return serial

    except:
        return ''

    return ''

def creatSqlite(product_label, product_name, serial, license_b, expiration, start_time):
    db_path = "/var/nfs/license.db"
    if os.path.exists(db_path):  
        os.remove(db_path)
    conn = sqlite.connect(db_path)
    c = conn.cursor()
    c.execute("PRAGMA cipher_compatibility=4;") 
    c.execute("PRAGMA key='foobar';") 
    c.execute("""CREATE TABLE IF NOT EXISTS LicenseKey (
        product_label VARCHAR(255), 
        product_name VARCHAR(255),
        serial VARCHAR(255), 
        license_b VARCHAR(255), 
        expiration VARCHAR(255),
        start_time VARCHAR(255), 
        offline_time VARCHAR(255), 
        recheck_duration VARCHAR(255), 
        recheck BOOLEAN, 
        PRIMARY KEY(product_label))""")

    current_datetime = datetime.now()
    offline_time = current_datetime.strftime("%Y%m%d%H%M%S%Z")
    offline_time = offline_time + "0"
    recheck_duration = ""
    sqldata = "INSERT INTO LicenseKey VALUES ('" + product_label + "','" + product_name + "','" + serial + "','" + license_b + "','" + expiration + "','" + start_time + "', '" + offline_time + "','" + recheck_duration + "','')"

    c.execute(sqldata)
    c.close()
    conn.commit()
    conn.close()

class RegisterHelp(object):
    ''' 产线激活
    '''
    def __init__(self):
        cmd_mount_dev = 'mount | grep -E "type[\t ]devtmpfs" || mount -t devtmpfs devtmpfs /dev'
        cmd_mount_proc = 'mount | grep "type proc" || mount -t proc proc /proc'
        cmd_mount_sys = 'mount | grep "type sysfs" || mount -t sysfs sysfs /sys'

        code, out = runCommand(cmd_mount_dev)
        if code:
            print('mount dev error! {}'.format(out))
        code, out = runCommand(cmd_mount_proc)
        if code:
            print('mount dev error! {}'.format(out))
        code, out = runCommand(cmd_mount_sys)
        if code:
            print('mount dev error! {}'.format(out))

        cmd = "ls  /sys/class/net/ | xargs -i ls -l /sys/class/net/{} | grep -iv virtual | awk -F '/' '{print $NF}' | head -1 | xargs -i cat /sys/class/net/{}/address"
        code, mac = runCommand(cmd)

        cmd = 'cat /sys/class/dmi/id/product_serial'
        code, sn = runCommand(cmd)

        cmd='lsblk -J -bno NAME,SERIAL,TYPE,MODEL,MOUNTPOINT'
        code, disk = runCommand(cmd)
        try:
            jObj = json.loads(disk)
            disk = parse_disk(jObj.get('blockdevices', {}))
        except Exception:
            print('error')
            pass
        self.mac = mac
        self.sn = sn
        self.disk = disk

        cmd = "awk '/^kms_addr/{print $3}' /tmp/register.ini"
        code, self.url = runCommand(cmd)
        cmd = "awk '/^serial/{print $3}' /tmp/register.ini"
        code, self.serial = runCommand(cmd)
        cmd = "awk '/^note/{print $3}' /tmp/register.ini"
        code, self.note = runCommand(cmd)
        cmd = "awk '/^label/{print $3}' /tmp/register.ini"
        code, self.label = runCommand(cmd)

        pass

    def register(self):
        res = False
        if not self.serial or len(self.serial) < 1:
            return

        if len(self.serial) < 5:
            return res

        self.url = "http://" + self.url + "/api/product/start-product-C/"
        data = {"serial": self.serial, "sn": self.sn, "disk": self.disk, "mac": self.mac, "note": self.note, "label": self.label}
        postdata = json.dumps(data)
        buffer = BytesIO()

        curl = pycurl.Curl()
        curl.setopt(pycurl.CONNECTTIMEOUT, 1)
        curl.setopt(pycurl.TIMEOUT, 1)
        curl.setopt(pycurl.MAXREDIRS, 1)
        curl.setopt(pycurl.FORBID_REUSE, 1)
        header = ["Content-Type:application/json"]
        curl.setopt(pycurl.HTTPHEADER, header)
        curl.setopt(pycurl.POSTFIELDS, postdata)
        curl.setopt(pycurl.URL, self.url)
        curl.setopt(pycurl.WRITEFUNCTION, buffer.write)
        try:
            curl.perform()
        except Exception as err:
            print('register error [{}], err: {}'.format(self.serial, err))
            print("except Exception")
            res = False
        else:
            response_data = buffer.getvalue().decode('utf-8')
            dir_path = '/var/nfs/'
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
            with open("/var/nfs/kms_register.log", 'w') as f:
                f.write(response_data)
                f.close()
            if "激活成功" in response_data:
                res = True
                parsed_data = json.loads(response_data)
                expiration = product_label = product_name = license_b = ""
                for key, value in parsed_data.items():
                    if key == "license_b":
                        license_b = value.replace("-", "")
                    if key == "license_b_info":
                        expiration = value.get("expiration")
                        product_label = value.get("product_label")
                        product_name = value.get("product_name")
                current_datetime = datetime.now()
                start_time = current_datetime.strftime("%Y%m%d")
                creatSqlite(product_label, product_name, self.disk, license_b, expiration, start_time)
        return res

if __name__ == '__main__':
    registerClient = RegisterHelp()
    res = registerClient.register()
    if res:
        print('注册成功！')
        sys.exit(0)
    else:
        print('注册失败！')
        sys.exit(-1)
